//=================================================================================================
/// \file:		dcf77.c
///
/// \date:		23.04.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//===================================================================================================
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include "DCF77_Auswertung.h"
#include "LCD240X064.h"
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// ZEIT- und Datumswerk
// Struct fr Systemzeit initialisieren, dieser speichert die Daten, welche genutzt werden
volatile struct timeAndDate systemTime = {0,0,0,SATURDAY,1,JANUARY,BASE_YEAR};	
// Struct fr Zeitkopie initialisieren, dieser speichert die Daten, welche Zwischenwerte darstellen
volatile struct timeAndDate copyTime = {0,0,0,SATURDAY,1,JANUARY,BASE_YEAR};
		
// SIGNALABTASTUNG
// Variable zhlt die Interrupts des Timers 2
volatile uint8_t timer1InterruptCounter = 0;
// Variable speichert den Bitstrom des DCF77-Signals
volatile bool dcf77SignalBuffer[NUMBER_OF_BITS];
// Variable speichert die Stelle im Bitstrom-Puffer, an der das nchste Bit gespeichert wird
volatile uint8_t dcf77BitCounter = 0;
// Variable speichert den zuletzt detektierten Signalpegel (dient zur Steuerung der Signal-LED)
// Auf High gesetzt, damit mindestens ein LOW-Pegel anliegen muss, bevor eine steigende Flanke erfasst wird
volatile bool dcf77LastSignalLevel = HIGH_LEVEL;
// Variable speichert die Anzahl der ersten positiven Flanken des Empfngermoduls nach dessen Aktivierung
volatile uint8_t dcf77FirstEdgesCounter = 0;
// Variable signalisiert den Beginn einer neuen Zeitbertragung (wir beim 59. Sekundenbit gesetzt)
volatile bool dcf77SignalStartFound = false;
// Variable wird einmalig bei Beginn der Signalabtastung gesetzt und setzt den Interruptcounter auf 0 
volatile bool dcf77ResetInterruptCounter = false;
// Variable zhlt die High-Phasen (10 ms-Abschnitte) innerhalb einer Sekunde
volatile uint8_t dcf77HighPulseCounter = 0;

// DATENAUSGABE
// Variable wird jede Sekunde in der ISR gelscht 
// und im Hauptprogramm nach einer AUsgabe auf dem LCD wieder gesetzt
volatile bool timeIsSetet = false;
// Variable steuert die Ausgabe der Statusmeldungen
// Jede Statusmeldung wird nur einmal ausgegeben
volatile uint8_t dcf77CurrentMessage = false;
//-------------------------------------------------------------------------------------------------
// Functions
//------------------------------------------------------------------------------------------------- 
//=== Function: Dcf77InitSignalBuffer =============================================================
///
/// @brief	Die Funktion initialisiert den Datenspeicher fr das DCF77-Signal
///
/// @param	void
///
/// @return	void
//================================================================================================
void Dcf77InitSignalBuffer(void)
{
	// Laufvariable
	uint8_t i = 0;
	// Array, in dem das DCF77-Signal gespeichert wird, initialisieren
	for (i=0;i<NUMBER_OF_BITS;i++)
	{
		// Alle Elemente zu 0 initialisieren
		dcf77SignalBuffer[i] = 0;
	}	
}
//=== Function: Dcf77Timer1Init ===================================================================
///
/// @brief	Die Funktion initialisiert den 16-Bit Timer 1 (CTC-Mode, Presacaler: 256)
///
/// @param	void
///
/// @return	void
//=================================================================================================
void Dcf77Timer1Init(void)
{
	// CTC-Mode, Prescaler: 64
	TCCR1B |= ((1<<WGM12)|(1<<CS11)|(1<<CS10));
	// Compare-Wert
	OCR1A = COMPARE_CAPTURE_VALUE;
	// Timerinterrupt (Comparematch) aktivieren
	TIMSK1 |= (1<<OCIE1A);
}
//=== Function: Dcf77Init =========================================================================
///
/// @brief	Die Funktion initialisiert alle Ein- und Ausgnge fr den Betrieb des DCF77-Empfngers
///
/// @param	void
///
/// @return	void
//================================================================================================
void Dcf77Init(void)
{
	// Ausgang fr die Kontroll-LED deklarieren
	DCF77_CHECKING_LED_DDR |= (1<<DCF77_CHECKING_LED_PIN_NUMBER);
	// Ausgang zum Schalten fr das DCF77-Modul deklarieren
	DCF77_MODULE_POWER_DDR |= (1<<DCF77_MODULE_POWER_PIN_NUMBER);
	// Pullupwiderstand fr die Datenleitung aktivieren
	DCF77_SIGNAL_PORT |= (1<<DCF77_SIGNAL_PIN_NUMBER);
	// Signalpuffer initialisieren
	Dcf77InitSignalBuffer();
	// Timer initialisieren
	Dcf77Timer1Init();
	// Interrupts global aktivieren
	sei();
}
//=== Function: Dcf77SwitchDcfModule ==============================================================
///
/// @brief	Die Funktion schaltet das Dcf77-Modul ein- bzw. aus
///
/// @param	bool status
///
/// @return	void
//=================================================================================================
void Dcf77SwitchDcfModule(bool status)
{
	// Einschalten
	if (status == SWITCH_ON)
	{
		// Modul einschalten
		DCF77_MODULE_POWER_PORT |= (1<<DCF77_MODULE_POWER_PIN_NUMBER);
	}
	// Ausschalten
	if (status == SWITCH_OFF)
	{
		// Modul ausschalten
		DCF77_MODULE_POWER_PORT &= ~(1<<DCF77_MODULE_POWER_PIN_NUMBER);
		// Signal-LED ausschalten
		// Je nach Zustand/ Strung des Signals ist diese evtl. noch an
		DCF77_CHECKING_LED_PORT &= ~(1<<DCF77_CHECKING_LED_PIN_NUMBER);
	}
}
//=== Function: Dcf77CalculateTimeAndDate =========================================================
///
/// @brief	Die Funktion berrechnet aus dem empfangenen Signal Zeit und Datum
///
/// @param	void
///
/// @return	void
//=================================================================================================	
void Dcf77CalculateTimeAndDate(void)
{
	// Sekunde auf 0 setzen, da die errechnete Zeit am Anfang einer Minute beginnt 
	copyTime.second = 0;
	// Die Zahlen im Bitstrom-Array entsprechen den Positionen der Bits im DCF77-Signal 
	// Minute berechnen
	copyTime.minute = ( (dcf77SignalBuffer[21]*BIT_SIGNIFICANCE_1)
											+(dcf77SignalBuffer[22]*BIT_SIGNIFICANCE_2)
											+(dcf77SignalBuffer[23]*BIT_SIGNIFICANCE_4)
							 		 		+(dcf77SignalBuffer[24]*BIT_SIGNIFICANCE_8)
								 			+(dcf77SignalBuffer[25]*BIT_SIGNIFICANCE_10)
											+(dcf77SignalBuffer[26]*BIT_SIGNIFICANCE_20)
											+(dcf77SignalBuffer[27]*BIT_SIGNIFICANCE_40) );
	// Stunde berechnen							
	copyTime.hour = ( (dcf77SignalBuffer[29]*BIT_SIGNIFICANCE_1)
									  +(dcf77SignalBuffer[30]*BIT_SIGNIFICANCE_2)
									  +(dcf77SignalBuffer[31]*BIT_SIGNIFICANCE_4)
										+(dcf77SignalBuffer[32]*BIT_SIGNIFICANCE_8)
										+(dcf77SignalBuffer[33]*BIT_SIGNIFICANCE_10)
										+(dcf77SignalBuffer[34]*BIT_SIGNIFICANCE_20) );
	// Wochentag berechnen
	copyTime.weekDay = ( (dcf77SignalBuffer[42]*BIT_SIGNIFICANCE_1)
											+(dcf77SignalBuffer[43]*BIT_SIGNIFICANCE_2)
											+(dcf77SignalBuffer[44]*BIT_SIGNIFICANCE_4) );
	// Monatstag berechnen
	copyTime.monthDay = ( (dcf77SignalBuffer[36]*BIT_SIGNIFICANCE_1)
											 +(dcf77SignalBuffer[37]*BIT_SIGNIFICANCE_2)
											 +(dcf77SignalBuffer[38]*BIT_SIGNIFICANCE_4)
											 +(dcf77SignalBuffer[39]*BIT_SIGNIFICANCE_8)
											 +(dcf77SignalBuffer[40]*BIT_SIGNIFICANCE_10)
											 +(dcf77SignalBuffer[41]*BIT_SIGNIFICANCE_20) );
	// Monat berechnen										
	copyTime.month = (  (dcf77SignalBuffer[45]*BIT_SIGNIFICANCE_1)
										 +(dcf77SignalBuffer[46]*BIT_SIGNIFICANCE_2)
										 +(dcf77SignalBuffer[47]*BIT_SIGNIFICANCE_4)
										 +(dcf77SignalBuffer[48]*BIT_SIGNIFICANCE_8)
										 +(dcf77SignalBuffer[49]*BIT_SIGNIFICANCE_10) );	
	// Jahr berechnen
	copyTime.year = (	 BASE_YEAR
										+(dcf77SignalBuffer[50]*BIT_SIGNIFICANCE_1)
										+(dcf77SignalBuffer[51]*BIT_SIGNIFICANCE_2)
									  +(dcf77SignalBuffer[52]*BIT_SIGNIFICANCE_4)
							 	 	  +(dcf77SignalBuffer[53]*BIT_SIGNIFICANCE_8)
							  	  +(dcf77SignalBuffer[54]*BIT_SIGNIFICANCE_10)
									  +(dcf77SignalBuffer[55]*BIT_SIGNIFICANCE_20)
									  +(dcf77SignalBuffer[56]*BIT_SIGNIFICANCE_40) );
}
//=== Function: Dcf77CheckTransmission ============================================================
///
/// @brief	Die Funktion berprft das empfangene Signal auf Fehler durch Vergelich der Prfbits
///
/// @param	void
///
/// @return	bool
//=================================================================================================
bool Dcf77CheckTransmission(void)
{	
	// Prfergebnis
	bool transmissionStatus = TRANSMISSION_ERROR;
	// Laufvariable
	uint8_t i = 0;
	// Summe der Minuten-Bits
	uint8_t sumMinutes = 0;
	// Summe der Stunden-Bits
	uint8_t sumHours = 0;
	// Summe der Datums-Bits
	uint8_t sumDate = 0;
	// Paritt der Minuten-Bits
	bool parityMinutes = 0;
	// Paritt der Stunden-Bits
	bool parityHours = 0;
	// Paritt der Datums-Bits
	bool parityDate = 0;
	// Aufsummieren der Minuten-Bits
	for (i=FIRST_BIT_MINUTE;i<=LAST_BIT_MINUTE;i++)
	{
		sumMinutes+= dcf77SignalBuffer[i]; 
	}
	// Aufsummieren der Stunden-Bits
	for (i=FIRST_BIT_HOUR;i<=LAST_BIT_HOUR;i++)
	{
		sumHours+= dcf77SignalBuffer[i]; 
	}
	// Aufsummieren der Datums-Bits
	for (i=FIRST_BIT_DATE;i<=LAST_BIT_DATE;i++)
	{
		sumDate+= dcf77SignalBuffer[i]; 
	}
	// Paritt der Minuten-Bits berechnen
	// Berechnung des Restes der Summe der Minuten-Bits bei der Division durch 2 
	// Ist der Rest 0, so ist die Summe gerade, ist er 1, so ist sie ungerade
	parityMinutes = (sumMinutes % MODULO_TWO);
	// Paritt der Stunden-Bits berechnen
	parityHours = (sumHours % MODULO_TWO);
	// Paritt der Datums-Bits berechnen
	parityDate = (sumDate % MODULO_TWO);
	// Vergleich der berechneten Parittsbits mit den bertragenen Parittsbits
	if ((parityMinutes == dcf77SignalBuffer[PARITY_BIT_MINUTES])
			&& (parityHours == dcf77SignalBuffer[PARITY_BIT_HOURES])
			&& (parityDate == dcf77SignalBuffer[PARITY_BIT_DATE]))
	{
		// Nur wenn alle errechneten Patittsbits mit den empfangen Prittsbits bereinstimmen
		// gilt die bertragung als fehlerfrei
		transmissionStatus = TRANSMISSION_OK;
	}
	// Ergebis zurckgeben
	return transmissionStatus;
}	
//=== Function: Dcf77CheckplausibilityOfData ======================================================
///
/// @brief	Die Funktion berprft die errechneten Daten auf Plausibilitt
///
/// @param	void
///
/// @return	bool
//=================================================================================================
bool Dcf77CheckplausibilityOfData(void)
{
	// Prfergebnis
	bool plausibilityStaus = PLAUSIBILITY_ERROR;	
	// Die errechneten Zeiten auf Plausibilitt berprfen
	if ((copyTime.minute <= LAST_MINUTE_OF_HOUR)
		 && (copyTime.hour <= LAST_HOUR_OF_DAY)
		 && (copyTime.weekDay <= SUNDAY)
		 && (copyTime.monthDay <= LATEST_MONTH_DAY_IN_YEAR)
		 && (copyTime.month <= DECEMBER))
	{
		// Nur wenn alle Daten plausibel sind, gelten die Daten als korrekt
		plausibilityStaus = PLAUSIBILITY_OK;
	}
	// Ergebis zurckgeben
	return plausibilityStaus;	
}
//=== Function: Dcf77WriteMessage =================================================================
///
/// @brief	Die Funktion gibt verschiedene Stausmeldungen ber die Zeiterfassung aus
///
/// @param	uint8_t status
///
/// @return	void
//=================================================================================================
void Dcf77WriteMessage(uint8_t status)
{
	// Alte Stausmeldung lschen
		LcdClear();
	// Je nach Status wird eine Infomeldung auf ein LCD ausgegeben
	switch (status)
	{
		// Empfngermodul wird aktiviert, warten auf Bereitschaft
		case WAIT_FOR_MODULE:
				// Nachricht ausgeben
				LcdWriteString("Warte auf Bereitschaft Empfangs-Modul",
											 FIRST_STATUS_MESSAGE_PART_A_LINE,
											 FIRST_STATUS_MESSAGE_PART_A_COLUMN);
				// Cursor setzen
				LcdSetCursor(CURSOR_POSITION_FIRST_STATUS_MESSAGE_LINE,
										 CURSOR_POSITION_FIRST_STATUS_MESSAGE_COLUMN);
				// Steuervariable setzen, damit das Hauptprogramm wei, welche Meldung ausgegeben wurde
				dcf77CurrentMessage = WAIT_FOR_MODULE;
				break;
		// Empfngermodul ist betriebsbereit, Signalanfang suchen
		case MODULE_IS_READY_SEARCH_SIGNAL:
				// Erste Nachricht ausgeben
				LcdWriteString("Empfangs-Modul betriebsbereit",
											 FIRST_STATUS_MESSAGE_PART_B_LINE,
											 FIRST_STATUS_MESSAGE_PART_B_COLUMN);
				// Zweite Nachricht ausgeben
				LcdWriteString("Suche Signalanfang DCF77-Signal",
											 SECOND_STATUS_MESSAGE_LINE,
											 SECOND_STATUS_MESSAGE_COLUMN);
				// Cursor setzen
				LcdSetCursor(CURSOR_POSITION_SECOND_STATUS_MESSAGE_LINE,
										 CURSOR_POSITION_SECOND_STATUS_MESSAGE_COLUMN);
				// Steuervariable setzen, damit das Hauptprogramm wei, welche Meldung ausgegeben wurde
				dcf77CurrentMessage = MODULE_IS_READY_SEARCH_SIGNAL;
				break;
		// Signalanfang gefunden, Signal wird abgetastet
		case MODULE_IS_READY_SIGNAL_FOUND_TIME_IS_UPDATING:
				// Erste Nachricht ausgeben
				LcdWriteString("Empfangs-Modul betriebsbereit",
											 FIRST_STATUS_MESSAGE_PART_B_LINE,
											 FIRST_STATUS_MESSAGE_PART_B_COLUMN);
				// Zweite Nachricht ausgeben
				LcdWriteString("Mit DCF77-Signal synchronisiert",
											 SECOND_STATUS_MESSAGE_LINE,
											 SECOND_STATUS_MESSAGE_COLUMN);
				// Dritte Nachricht ausgeben
				LcdWriteString("Uhrzeit wird aktualisiert",
											 THIRD_STATUS_MESSAGE_LINE,
											 THIRD_STATUS_MESSAGE_COLUMN);
				// Cursor setzen
				LcdSetCursor(CURSOR_POSITION_THIRD_STATUS_MESSAGE_LINE,
										 CURSOR_POSITION_THIRD_STATUS_MESSAGE_COLUMN);
				// Steuervariable setzen, damit das Hauptprogramm wei, welche Meldung ausgegeben wurde
				dcf77CurrentMessage = MODULE_IS_READY_SIGNAL_FOUND_TIME_IS_UPDATING;
				break;
	}	
}
//=== Function: ISR(TIMER1_COMPA_vect) =============================================================
///
/// @brief	Interrupt-Servisce-Routine des Timer 2, wird alle 10 Millisekunden aufgerufen
///					Mit dem Begriff "Signal" ist in den Komentaren dieser ISR immer das DCF77-Signal gemeint
///					
/// @param	void
///
/// @return	void
//=================================================================================================
ISR(TIMER1_COMPA_vect)
{
	// Interruptzhler bei jedem Durchlauf der ISR um 1 erhhen
	timer1InterruptCounter++;
	// Das DCF77-Signal wird nur abgetastet, wenn das Empfngermodul eingeschaltet ist
	if (DCF77_MODULE_POWER_PIN & (1<<DCF77_MODULE_POWER_PIN_NUMBER))
	{
		// Die Signal-LED ist ein Indikator fr das vom Empfngermodul ausgegebene digitale Signal
		// Die LED ist solange aktiv, wie das Modul eingeschaltet ist
		// Variable fr aktuellen Signalpegel anlegen und Pegel einlesen
		bool dcf77CurrentSignalLevel = (DCF77_SIGNAL_PIN & (1<<DCF77_SIGNAL_PIN_NUMBER));
		// Kontroll-LED bei jeder steigenden Flanke toggeln
		if ((dcf77LastSignalLevel == LOW_LEVEL)
			 && (dcf77CurrentSignalLevel == HIGH_LEVEL))
		{
			// Kontroll-LED toggeln
			DCF77_CHECKING_LED_PORT ^= (1<<DCF77_CHECKING_LED_PIN_NUMBER);
			// Die Anzahl der positiven Flanken werden in der Variablen "dcf77FirstEdgesCounter" gespichert
			// Diese wird zu Beginn der Zeiterfassung auf 0 gesetzt 
			// Sie dient der Ermittlung des Statuses des Empngermoduls und wird bis max. 3 hochgezhlt
			// Nach 3 erkannten positiven Flanken gilt das Empngermodul als betriebsbereit
			if (dcf77FirstEdgesCounter < NUMBER_OF_EDGES_UNTIL_MODULE_IS_READY)
			{
				// Flankenzhler um 1 erhhen
				dcf77FirstEdgesCounter++;
			}
		}
		// Den zu Beginn der ISR erfassten Signalpegel speichern
		dcf77LastSignalLevel = dcf77CurrentSignalLevel;
		// Wenn das Empfngermodul betriebsbereit ist, wird das Signal ausgewertet
		if (dcf77FirstEdgesCounter == NUMBER_OF_EDGES_UNTIL_MODULE_IS_READY)  
		{
			// Bei erstmaligen Eintritt in die Signalauswertung wird der Interrutzhler auf 0 gesetzt,
			// damit die Pulse von Beginn an der Sekunde gezhlt werden
			if (dcf77ResetInterruptCounter)
			{
				timer1InterruptCounter = 0;
				// Steuervariable auf false setzten, damit der Interruptcounter nur einmal zu 0 gesetzt wird
				dcf77ResetInterruptCounter = false;
			}
			// Signalabtastung: alle 10 ms wird das Signal abgetastet
			// Die Variable "dcf77HighPulseCounter" zhlt die Anzahl der 10 ms-Abschnitte, in denen das Signal
			// einen High-Pegel hat
			if (dcf77CurrentSignalLevel)
			{
				// Bei jeem High-Pegel die Variable um 1 erhhen
				dcf77HighPulseCounter++;
			}
			// Auswertung der erfassten High-Pegel erfolgt nach jeder Sekunde
			if (timer1InterruptCounter == INTERRUPTS_PER_SECOND)
			{
				// Wenn der Signalanfang gefunden ist, werden die High-Phasen darufhin untersucht,
				// ob es sich um eine logische 0 (100 ms lang) oder eine logische 1 (200 ms lang) handelt
				if (dcf77SignalStartFound)
				{
					// Wenn die erfasste High-Phase zwischen 70 ms und 130 ms liegt, 
					// gilt das empfangene Bit als eine 0
					if ((dcf77HighPulseCounter >= MINIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_0)
						 && (dcf77HighPulseCounter <= MAXIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_0))
					{
						// Eine logische 0 im Puffer abspeichern
						dcf77SignalBuffer[dcf77BitCounter] = LOW_LEVEL;
					}
					// Wenn die erfasste High-Phase zwischen 170 ms und 230 ms liegt,
					// gilt das empfangene Bit als eine 1
					if ((dcf77HighPulseCounter >= MINIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_1)
						 && (dcf77HighPulseCounter <= MAXIMUM_NUMBER_OF_PULSES_FOR_LOGICAL_1))
					{
						// Eine logische 1 im Puffer abspeichern
						dcf77SignalBuffer[dcf77BitCounter] = HIGH_LEVEL;
					}
					// Den Bitcounter nach jedem im Bitstrom-Puffer gespeichertem Bit um 1 erhhen,
					// damit beim nchsten empfangenen Bit an die nchste Stelle im Puffer geschreiben wird
					dcf77BitCounter++;
				}
				// Wenn innerhalb einer Sekunde maximal 30 ms ein High-Pegel anliegt, entspricht
				// diese Sekunde der 59. Sekunde und damit dem Beginn der neuen Zeitbertragung
				// 30 ms sind als Strungen anzusehen, im Idealfall sind es 0 ms
				// Der Startmarker (dcf77SignalStartFound) wird nach der Signalauswertung (oberer Block) gesetzt,
				// weil erst mit der nchsten Sekunde die Zeitsignalbertragung beginnt
				if (dcf77HighPulseCounter <= MAXIMUM_NUMBER_OF_PULSES_FOR_STARTMARK)
				{
					dcf77SignalStartFound = true;
				}	
				// Highpegel-Zhler zurcksetzen
				dcf77HighPulseCounter = 0;
			}
		}	
	}
	// Der folgende Programmteil dient der nderung der Zeitvariablen und der Erkennung des Signalanfangs
	// Jeder 100. Interruptaufruf entspricht exakt einer Sekunde (100 * 10ms = 1000ms) 
	// Das Zhlwerk fr die Zeit ist daher so gekapselt, dass es nur bei jedem 100. Aufruf der ISR durchlaufen wird
	if (timer1InterruptCounter == INTERRUPTS_PER_SECOND)
	{	
		// Ab hier werden die Sekunden, Minuten, Stunden etc. gesetzt
		// Dies ist unabhngig von der Erfassung des DCF77-Signals
		// SEKUNDEN SETZEN
		// Wenn der Sekundenwert unter 60 ist, wird der Wert um 1 erhht
		if (systemTime.second < LAST_SECOND_OF_MINUTE)
		{
			// Sekundenwert um 1 erhhen
			systemTime.second++;
		}
		// Wenn 60 Sekunden (1 Minute) verstrichen sind, wird der Sekundenwert auf 0 gesetzt
		else
		{
			// Sekundenwert auf 0 setzen
			systemTime.second = 0;
		}
		// MINUTEN SETZEN
		// Wenn 60 Sekunden verstrichen sind und der Minutenwert unter 59 liegt, wird der Wert um 1 erhht
		if (systemTime.second == 0)
		{
			if (systemTime.minute < LAST_MINUTE_OF_HOUR)
			{
				// Minutenwert um 1 erhhen
				systemTime.minute++;
			}
			// Wenn 60 Minuten verstrichen sind, wird der Minutenwert auf 0 gesetzt
			else
			{
				// Minutenwert auf 0 setzen
				systemTime.minute = 0;
			}
		}
		// STUNDEN SETZEN
		// Wenn 60 Minuten verstrichen sind und der Stundenwert unter 23 liegt, wird der Wert um 1 erhht
		if ((systemTime.second == 0)
			 && (systemTime.minute == 0))
		{
			if (systemTime.hour < LAST_HOUR_OF_DAY)
			{
				// Stundenwert um 1 erhhen
				systemTime.hour++;				
			}
			// Wenn 24 Stunden verstrichen sind, wird der Stundenwert auf 0 gesetzt
			else
			{
				systemTime.hour = 0;
			}
		}
		// WOCHENTAG SETZEN
		// Wenn 1 Tag verstrichen ist und der Wochentagwert unter 7 liegt, wird der Wert um 1 erhht
		if ((systemTime.second == 0)
			 && (systemTime.minute == 0)
			 && (systemTime.hour == 0))
		{
			if (systemTime.weekDay < SUNDAY)
			{
				// Wochentagwert um 1 erhhen
				systemTime.weekDay++;
			}
			// Wenn 7 Tage verstrichen sind, wird der Wochentagwert auf 1 (Montag) gesetzt
			else
			{
				// Wochentagwert auf 1 (Montag) setzen
				systemTime.weekDay = MONDAY;
			}
		}
		// MONATSTAG SETZEN
		// Je nach Monat wird unterschieden, nach wie vielen Tagen der Monatstagwert auf 0 gesetzt wird
		if ((systemTime.second == 0)
			 && (systemTime.minute == 0)
			 && (systemTime.hour == 0))
		{
			// Alle in der if-Abfrage aufgefhrten Monate haben 31 Tage
			// Wenn 1 Tag verstrichen ist und der Monatstagwert unter 31 liegt, wird der Wert um 1 erhht
			if ((systemTime.month == JANUARY)
				 || (systemTime.month == MARCH)
				 || (systemTime.month == MAY)
				 || (systemTime.month == JULY)
				 || (systemTime.month == AUGUST)
				 || (systemTime.month == OCTOBER)
				 || (systemTime.month == DECEMBER))
			{
				if (systemTime.monthDay < LAST_DAY_OF_MONTH_31)
				{
					// Monatstagwert um 1 erhhen
					systemTime.monthDay++;
				}			
				// Wenn 1 Monat verstrichen ist, wird der Monatstagwert auf 1 gesetzt
				else
				{
					// Monatstagwert auf 1 setzen
					systemTime.monthDay = 1;
				}
			}
			// Alle in der if-Abfrage aufgefhrten Monate haben 30 Tage
			// Wenn 1 Tag verstrichen ist und der Monatstagwert unter 30 liegt, wird der Wert um 1 erhht
			if ((systemTime.month == APRIL)
				 || (systemTime.month == JUNE)
				 || (systemTime.month == SEPTEMBER)
				 || (systemTime.month == NOVEMBER))
			{
				if (systemTime.monthDay < LAST_DAY_OF_MONTH_30)
				{
					// Monatstagwert um 1 erhhen
					systemTime.monthDay++;
				}			
				// Wenn 1 Monat verstrichen ist, wird der Monatstagwert auf 1 gesetzt
				else
				{
					// Monatstagwert auf 1 setzen
					systemTime.monthDay = 1;
				}
			}
			// Der Februar hat je nach dem, ob ein Schaltjahr vorliegt, 28 oder 29 Tage
			// Wenn 1 Tag verstrichen ist und der Monatstagwert unter 28 bzw. 29 (wenn Schaltjahr) liegt, wird der Wert um 1 erhht
			if (systemTime.month == FEBRUARY)
			{
				// berprfen, ob ein Schaltjahr vorliegt
				// Schaltjahr
				if ((systemTime.year%4) == 0)
				{
					// Wenn 1 Tag verstrichen ist und der Monatstagwert unter 29 liegt, wird der Wert um 1 erhht
					if (systemTime.monthDay < LAST_DAY_OF_MONTH_29)
					{
						// Monatstagwert um 1 erhhen
						systemTime.monthDay++;
					}
					// Wenn 1 Monat verstrichen ist, wird der Monatstagwert auf 1 gesetzt
					else
					{
						// Monatstagwert auf 1 setzen
						systemTime.monthDay = 1;
					}
				}
				// Kein Schaltjahr
				else
				{
					// Wenn 1 Tag verstrichen ist und der Monatstagwert unter 28 liegt, wird der Wert um 1 erhht
					if (systemTime.monthDay < LAST_DAY_OF_MONTH_28)
					{
						// Monatstagwert um 1 erhhen
						systemTime.monthDay++;
					}
					// Wenn 1 Monat verstrichen ist, wird der Monatstagwert auf 1 gesetzt
					else
					{
						// Monatstagwert auf 1 setzen
						systemTime.monthDay = 1;
					}
				}
			}
		}
		// MONAT SETZEN
		// Wenn 28, 29, 30 odr 31 Tage verstrichen sind und der Monatswert unter 12 (Dezember) liegt, wird der Wert um 1 erhht 
		if ((systemTime.second == 0)
			 && (systemTime.minute == 0)
			 && (systemTime.hour == 0)
			 && (systemTime.monthDay == 1))
		{
			if (systemTime.month < DECEMBER)
			{
				// Monatswert um 1 erhhen
				systemTime.month++;
			}
			// Monatswert auf 1 (Januar) setzen
			else
			{
				systemTime.month = JANUARY;
			}
		}
		// JAHRE SETZEN
		// Wenn 1 Monat verstrichen ist wird der Jahreswert um 1 erhht
		if ((systemTime.second == 0)
			 && (systemTime.minute == 0)
			 && (systemTime.hour == 0)
			 && (systemTime.monthDay == 1)
			 && (systemTime.month == JANUARY))
		{
			systemTime.year++;
		}
		// Interruptzhler auf 0 zurcksetzen, damit erst nach 125 Aufrufen der ISR (= 1 Sekunde)
		// das Zeitwerk wieder durchlaufen wird
		timer1InterruptCounter = 0;
		// Variable zur Steuerung der Ausgabe der Daten auf dem LCD zurcksetzen
		timeIsSetet = false;
	}	
}