//=================================================================================================
/// \file:		LCD240X064.c
///
/// \date:		16.07.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include "LCD240X064.h"
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Variable zur Speicherung des dezimalen Wertes einer komplett gesetzten Bitreihe
// in Abhngigkeit von der Zeichngre
uint8_t setAllBitsOfFont = (FONT_SIZE == 6)?0x3F:0xFF;		
// Bitmaps benutzerdefimierter Zeichen
// Grad-Zeichen
uint8_t bitmapDegreeSign[HEIGHT_SPECIFIC_SIGN] = {0x0E,0x0A,0x0E,0x00,0x00,0x00,0x00,0x00};																									
//-------------------------------------------------------------------------------------------------
// Functions
//------------------------------------------------------------------------------------------------
//=== Function: LcdEnablePulse ====================================================================
///
/// @brief	Funktion erzeugt einen Enable-Impuls auf der Chip-Enable-Leitung
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdEnablePulse(void)
{
	// Displaycontroller ansprechen, Enable-Leitung auf Low legen (chip enable ist low-aktiv)
	CONTROL_PORT &= ~(1<<ENABLE);
	// Pegel eine definierte Zeit (laut Datenblatt) anliegen lassen
	_delay_us(ENABLE_TIME_US);
	// Displaycontroller abwhlen, Enable-Leitung auf High legen
	CONTROL_PORT |= (1<<ENABLE);
}
//=== Function: LcdReset ==========================================================================
///
/// @brief	Funktion erzeugt einen Reset-Impuls auf der Reset-Leitung
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdReset(void)
{
	// Resetleitung auf Low legen (Reset ist low-aktiv)
	CONTROL_PORT &= ~(1<<RESET);
	// Pegel eine definierte Zeit (laut Datenblatt) anliegen lassen
	_delay_us(RESET_TIME_US);
	// Resetleitung auf Hig legen (Reset beenden)
	CONTROL_PORT |= (1<<RESET);
}
//=== Function: LcdCheckStatus ====================================================================
///
/// @brief	Funktion berprft die Statusbits STA0 und STA1 und wartet solange, bis beide 1 sind
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdCheckStatus(void)
{
	// Datenbus als Eingang setzen
	DATA_DDR = 0x00;
	// Command/Data auf high legen, Write deaktivieren
	CONTROL_PORT |= ((1<<COMMAND_DATA)|(1<<WRITE));
	// Read deaktivieren und Enable auf low legen (Controller anwhlen)
	CONTROL_PORT &= ~((1<<ENABLE)|(1<<READ));
	// Solange warten, bis die Status-Bits STA0 und STA1 gesetzt sind
	while (!((DATA_PIN & (0x03)) == 0x03))
	{
	}
	// Read deaktivieren und Enable auf high legen
	CONTROL_PORT |= ((1<<ENABLE)|(1<<READ));
	// Datenbus als Ausgang zurckdefinieren
	DATA_DDR = 0xFF;
}
//=== Function: LcdSendCommand ====================================================================
///
/// @brief	Funktion sendet einen Befehl an das LCD
///
/// @param	unsigned char, Befehl
///
/// @return	void
//=================================================================================================
void LcdSendCommand(unsigned char command)
{
	// Status prfen
	LcdCheckStatus();
	// Daten auf den Datenbus legen
	DATA_PORT = command;
	// Command auf high setzen
	CONTROL_PORT |= ((1<<COMMAND_DATA)|(1<<READ));
	// Write aktivieren
	CONTROL_PORT &= ~(1<<WRITE);
	// Enable-Impuls
	LcdEnablePulse();
	// Write wieder deaktivieren
	CONTROL_PORT |= (1<<WRITE);
}
//=== Function: LcdSendData =======================================================================
///
/// @brief	Funktion sendet ein Datenbyte an das LCD
///
/// @param	unsigned char, Daten
///
/// @return	void
//=================================================================================================
void LcdSendData(unsigned char data)
{
	// Status prfen
	LcdCheckStatus();
	// Daten auf den Datenbus legen
	DATA_PORT = data;
	// Command auf high setzen
	CONTROL_PORT |= (1<<READ);
	// Write aktivieren
	CONTROL_PORT &= ~((1<<COMMAND_DATA)|(1<<WRITE));
	// Enable-Impuls
	LcdEnablePulse();
	// Write wieder deaktivieren
	CONTROL_PORT |= (1<<WRITE);
}
//=== Function: LcdWriteData ======================================================================
///
/// @brief	Funktion sendet ein Datenbyte und inkrementiert den Adresspointer
///
/// @param	unsigned char, Daten
///
/// @return	void
//=================================================================================================
void LcdWriteData(unsigned char data)
{
	// Daten senden
	LcdSendData(data);
	// Befehl: Daten Schreiben und Adresspointer Inkrementieren
	LcdSendCommand(0xC0);
}
//=== Function: LcdClearRam =======================================================================
///
/// @brief	Funktion lscht den Inhalt des Grafik- und des Text-RAM
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdClear(void)
{
	// Textspeicherbereich lschen
	LcdClearText();
	// Grafikspeicherbereich lschen
	LcdClearGraphic();
}
//=== Function: LcdClearTextRam =====================================================================
///
/// @brief	Funktion lscht den Inhalt des Text-RAM
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdClearText(void)
{
	// Laufvariable
	uint16_t i = 0;
	// Adresspointer auf die erste Stelle des Textspeicherbereiches setzen
	LcdSetAddressPointer(HOME_ADRESS_TEXT);
	// Alle Elemente des Textspeicherbereiches auf 0 setzen
	for(i=0;i<TEXT_AREA;i++)
	{
		LcdWriteData(INITIAL_VALUE);
	}
}
//=== Function: LcdClearTextPartical ==============================================================
///
/// @brief	Funktion lscht den Inhalt des Text-RAM im bergebenen Spaltenintervall
///
/// @param	uint16_t Startadresse, uint16_t Endadresse
///
/// @return	void
//=================================================================================================
void LcdClearTextPartically(uint8_t line, 
														uint8_t startColumn, 
														uint8_t endColumn)
{
	// Laufvariable
	uint16_t i = 0;
	// Startadresse
	uint16_t addresStart = (  ((line-1)*COLUMNS_OF_DISLPLAY)
														+(startColumn-1)						   );
	// Endadresse
	uint16_t addresEnd = (	((line-1)*COLUMNS_OF_DISLPLAY)
												  +(endColumn-1)							   );
	// Adresspointer auf die gewhlte Startadresse des Textspeicherbereiches setzen
	LcdSetAddressPointer(addresStart);
	// Alle Elemente des Textspeicherbereiches bis zur Endadresse lschen
	for(i=0;i<=(addresEnd-addresStart);i++)
	{
		LcdWriteData(INITIAL_VALUE);
	}	
}
//=== Function: LcdClearGraphicRam ================================================================
///
/// @brief	Funktion lscht den Inhalt des Grafik-RAM
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdClearGraphic(void)
{
	// Laufvariable
	uint16_t i = INITIAL_VALUE;
	// Adresspointer auf die erste Stelle des Grafikspeicherbereiches setzen
	LcdSetAddressPointer(HOME_ADRESS_GRAPHIC);
	// Alle Elemente des Grafikspeicherbereiches auf 0 setzen
	for(i=0;i<GRAPHIC_AREA;i++)
	{
		LcdWriteData(INITIAL_VALUE);																																											
	}
}
//=== Function: LcdInit ===========================================================================
///
/// @brief	Funktion initialisiert das LCD (Speicheraufteilung, Diyplaymode, Zeichendarstellung)
///
/// @param	void
///
/// @return	void
//=================================================================================================
void LcdInit(void)
{
	// Datenbus als Ausgang setzen
	DATA_DDR = 0xFF;
	// Datenbus komplett auf low ziehen 
	DATA_PORT = 0x00;
	// Steuerleitungen als Ausgang setzen
	CONTROL_DDR  |= ((1<<WRITE)|(1<<READ)|(1<<ENABLE)|(1<<COMMAND_DATA)|(1<<RESET));
	// Alle steuerleitungen auf high ziehen:
	// Write, Read, Enable und Reset nicht aktiv, Command/Data auf Command/Status gesetzt
	CONTROL_PORT |= ((1<<WRITE)|(1<<READ)|(1<<ENABLE)|(1<<COMMAND_DATA)|(1<<RESET));
	// Reset ausfhren
	LcdReset();
	// Die Hex-Zahlen der command-Funktionen sind im Datenblatt des Displays hinterlegt
	// Textspeicherbereich setzen:
	// Startadresse setzen
	LcdSendData(HOME_ADRESS_TEXT & 0xFF);
	LcdSendData(HOME_ADRESS_TEXT>>BITSIZE_OF_ONE_BYTE);
	LcdSendCommand(SET_TEXT_HOME_ADDRESS);
	// Spaltenanzahl setzen
	LcdSendData(COLUMNS_OF_DISLPLAY);
	LcdSendData(0x00);
	LcdSendCommand(SET_TEXT_AREA);
	// Grafikspeicherbereich setzen:
	// Startadresse setzen
	LcdSendData(HOME_ADRESS_GRAPHIC & 0xFF);
	LcdSendData(HOME_ADRESS_GRAPHIC>>BITSIZE_OF_ONE_BYTE);
	LcdSendCommand(SET_GRAPHIC_HOME_ADDRESS);
	// Spaltenanzahl setzen
	LcdSendData(COLUMNS_OF_DISLPLAY);																																																				
	LcdSendData(EXOR_MODE);
	LcdSendCommand(SET_GRAPHIC_AREA);
	// Mode setzen:
	// Interner Zeichengenerator, EXOR-Mode
	LcdSendCommand(EXOR_MODE);
	// Display an, Cursor an, Cursorblinken aus, Text und Grafik an
	LcdSendCommand(LCD_CURSOR_TEXT_GRAPHIC_ON);
	// Cursor: 8 Pixelreihen hoch
	LcdSendCommand(CURSOR_PATERN_8);
	// RAM reinigen
	LcdClear();
}
//=== Function: LcdSetAddressPointer ==============================================================
///
/// @brief	Funktion setzt den Adresspointer an eine bestimmte Position (Parameter: Adresse)
///
/// @param	uint16_t, Adresse
///
/// @return	void
//=================================================================================================
void LcdSetAddressPointer(uint16_t address)
{
	// Variablen fr LSB und MSB
	// Das niederwertigste Byte von "adress" maskieren und dem LSB zuweisen
	uint8_t addressLsb = (address & 0xFF);
	// "adress" um 8 Stellen (1 Byte) nach rechts shiften und MSB zuweisen
	uint8_t addressMsb = (address >> BITSIZE_OF_ONE_BYTE);
	// LSB und MSB senden
	LcdSendData(addressLsb);
	LcdSendData(addressMsb);
	// Befehl: Adresspointer setzen
	LcdSendCommand(SET_ADDRESS_POINTER);
}
//=== Function: LcdJumpToPosition =================================================================
///
/// @brief	Funktion setzt den Adresspointer an eine bestimmte Position (Parameter: Zeile und Spalte)
///
/// @param	uint8_t, Zeile; uint8_t, Spalte
///
/// @return	void
//=================================================================================================
void LcdJumpToPosition(uint8_t line, 
											 uint8_t column)
{
	// Adresse fr den Adresspointerausrechnen
	uint16_t address = ( HOME_ADRESS_TEXT
											+((line-1)*COLUMNS_OF_DISLPLAY)
											+(column-1)									   );
	// Adresse setzen
	LcdSetAddressPointer(address);
}
//=== Function: LcdSwitchCursor ===================================================================
///
/// @brief	Funktion schaltet den Cursor ein oder aus
///
/// @param	bool status
///
/// @return	void
//=================================================================================================
void LcdSwitchCursor(bool status)
{
	// Cursor einschalten
	if (status == SWITCH_ON)
	{
		// Display an, Cursor an, Cursorblinken an, Text und Grafik an
		LcdSendCommand(LCD_CURSORBLINK_TEXT_GRAPHIC_ON);
	}	
	// Cursor ausschalten
	if (status == SWITCH_OFF)
	{
		// Display an, Cursor an, Cursorblinken aus, Text und Grafik an
		LcdSendCommand(LCD_CURSOR_TEXT_GRAPHIC_ON);		
	}
}
//=== Function: LcdSetCursor ======================================================================
///
/// @brief	Funktion setzt den Cursor an eine bestimmte Position (Parameter: Zeile und Spalte)
///
/// @param	uint8_t line, uint8_t column
///
/// @return	void
//=================================================================================================
void LcdSetCursor(uint8_t line,
									uint8_t column)
{
	// Bei Aufruf der Funktion, das Cursorblinken aktivieren
	// Display an, Cursor an, Cursorblinken an, Text und Grafik an
	LcdSendCommand(LCD_CURSORBLINK_TEXT_GRAPHIC_ON);
	// Spalten-Position senden
	LcdSendData(column-1);
	// Zeilen-Position senden
	LcdSendData(line-1);
	// Befehl: Cursor setzen
	LcdSendCommand(SET_CURSOR);
}
//=== Function: LcdWriteChar ======================================================================
///
/// @brief	Funktion senden ein Zeichen (mit Adresspointer-Inkrementierung)
///
/// @param	unsigned char, Buchstabe/ Zeichen
///
/// @return	void
//=================================================================================================
void LcdWriteChar(unsigned char character)
{
	// Buchstaben sind mit einem Offset von 32 (0x20) coniert
	LcdWriteData(character-ASCII_OFFSET);
}
//=== Function: LcdWriteString ====================================================================
///
/// @brief	Funktion sendet zeichenweise einen String an das LCD
///
/// @param	unsigned char, Textstring
///
/// @return	void
//=================================================================================================
void LcdWriteString(char string[],
										uint8_t line, 
										uint8_t column)
{
	// Laufvariable
	uint8_t i = 0;
	// Zhlvariable
	uint8_t n = 0;
	// Zur Position springen, an der geschrieben werden soll
	LcdJumpToPosition(line,
										column);
	// Textlnge bestimmen
	n = strlen(string);
	// Textarray in einer Schleife ausgeben
	for(i=0;i<n;i++)
	{
		// Umlaute rausfiltern und gesondert ausgeben
		// Die Hex-Zahlen der Umlaute sind im Datenblatt des Displays hinterlegt
		switch (string[i])
		{
			case '':
				LcdWriteData(CODE_AE_SMALL);
				break;
			case '':
				LcdWriteData(CODE_AE_LARGE);
				break;
			case '':
				LcdWriteData(CODE_OE_SMALL);
				break;
			case '':
				LcdWriteData(CODE_OE_LARGE);
				break;
			case '':
				LcdWriteData(CODE_UE_SMALL);
				break;
			case '':
				LcdWriteData(CODE_UE_LARGE);
				break;
			// Wenn Zeichen kein Umlaut ist: Ausgabe mit LcdWriteChar
			default:
				LcdWriteChar(string[i]);
				break;
		}
	}
}
//=== Function: LcdWriteInteger ===================================================================
///
/// @brief	Funktion gibt eine Zahl dezimal auf dem Diyplay aus whlbarer Zahlenbreite
///
/// @param	int16_t value, uint8_t size, bool statusOfSign, uint8_t line, uint8_t column
///
/// @return	void
//=================================================================================================
void LcdWriteInteger(int16_t value, 
										 uint8_t size,
										 bool statusOfSign, 
										 uint8_t line, 
										 uint8_t column)
{
	// An die Position springen, an die geschrieben werden soll
	LcdJumpToPosition(line,column);
	// Wenn Vorzeichen ausgegeben werden soll
	if (statusOfSign == SIGN)
	{
		// Wenn der Wert negativ ist, wird ein Minuszeichen ausgegeben
		if (value < 0)
		{
			LcdWriteString("-",
										 line,
										 column);
		}
		// Wenn der Wert positiv ist, wird ein Pluszeichen ausgegeben
		else
		{
			LcdWriteString("+",
										 line,
										 column);
		}
		// Spaltenposition um 1 erhhen, 
		//damit die nachfolgenden Ziffern eine Zeichenbreite weiter rechts ausgegeben werden
		column++;
	}
	// Wenn die Zahl negativ ist
	if (value < 0)
	{
		// Zahl invertieren, da untere Routine kein 2-Komplement ausgeben kann
		value = -value;
	}
	// "size" gibt die Anzahl der Ziffern der Zahl "value" an
	// Je nach dem, wie viele Stellen die Zahl hat, werden entsprechend viele Ziffern ausgegeben (inklusive Nullen)
	// Da bei der switch-Selektion das break fehlt, werden alle Befehle ab dem ersten angesprungenden Befehl ausgefhrt
	// 48 ist der ASCI-Code fr 0 
	switch (size)
	{
		case 5:	
			LcdWriteChar(  ASCII_CODE_FOR_ZERO
									  +(value/10000)%10    );
		case 4:	
			LcdWriteChar(  ASCII_CODE_FOR_ZERO
									  +(value/1000)%10     );
		case 3:	
			LcdWriteChar(  ASCII_CODE_FOR_ZERO
										+(value/100)%10		   );
		case 2:	
			LcdWriteChar(  ASCII_CODE_FOR_ZERO
									  +(value/10)%10		   );
		case 1:	
			LcdWriteChar(  ASCII_CODE_FOR_ZERO
										+value%10					   );
	}
}
//=== Function: LcdWriteDouble ====================================================================
///
/// @brief	Die Funktion gibt ein Fliekommazahl mit max. 3 Vor- und 3 Nachkommastellen aus
///
/// @param	double value, uint8_t sizePrePoint, uint8_t sizeAfterPoint, uint8_t line,
///					uint8_t column
///
/// @return	void
//=================================================================================================
void LcdWriteDouble(double value,
										uint8_t sizePrePoint,
										uint8_t sizeAfterPoint,
										uint8_t line,
										uint8_t column)
{
	// Laufvariable
	uint8_t i = 0;
	// Multiplikationsfaktor
	uint16_t factor = 1;
	// Vorkommaanteil
	int16_t prePointValue = value;
	// Nachkommanteil
	uint32_t afterPointValue = 0;
	// Sicherstellen, das max. 4 Nachkommastellen ausgegeben werden
	if (sizeAfterPoint > MAX_SIZE_OF_AFTERPOINT)
	{
		sizeAfterPoint = MAX_SIZE_OF_AFTERPOINT;
	}
	// Faktor fr die Multiplikation mit dem Nachkommaanteil berechnen
	for (i=0;i<sizeAfterPoint;i++)
	{
		factor *= 10;
	}
	// Durch die Multiplikation von "factor" mit dem Nachkommaanteil von "value"
	// wird dieser so erhht, das er in einem Integer gespeichert werden kann
	// Die Addition von OFFSET_FOR_CORRECTION (0,00001) dient dazu, 
	// die Ungenauigkeiten der FLiekommazahl zu kompensieren
	afterPointValue = (value-(int16_t)value)*factor+OFFSET_FOR_CORRECTION;
	// Vorkommaanteil ausgeben 
	LcdWriteInteger(prePointValue,
									sizePrePoint,
									SIGN,
									line,
									column);
	// Komma ausgeben
	LcdWriteString(",",
								 line,
								 column+1+sizePrePoint);
	// Nachkommanteil ausgeben
	LcdWriteInteger(afterPointValue,
									sizeAfterPoint,
									NO_SIGN,
								  line,
								  column+2+sizePrePoint);
}
//=== Function: LcdWriteBinaryNumber ==============================================================
///
/// @brief	Funktion gibt eine Zahl an die bergebene osition binr auf dem Display aus
///
/// @param	uint8_t value, uint8_t size, uint8_t line, uint8_t column
///
/// @return	void
//=================================================================================================									 
void LcdWriteBinaryNumber(uint16_t value,
													uint8_t size,
													uint8_t line,
													uint8_t column)
{
	// Laufvariable fr Schleifen
	uint8_t i = 0;
	// Array fr das binre Ergbnis
	bool result [SIZE_16_BIT] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
	// Zwischenergebnis zur Berechnung der Binrzahl
	uint16_t quotient = value;
	// Berechnung durchfhren:
	// In der While-Schleife wird solange der bergebene Wert durch 2 geteilt, bis der Rest 0 ist
	while(quotient != 0)
	{
		// Der Rest der Division durch 2 ergibt das entsprechende Bit 
		result[i] = (quotient%2);
		// i gibt die Bitnumer an (0 bis 7), bei jedem Schleifendurchgang wird sie um 1 erhht 
		i++;
		quotient /= 2;
	}
	// Ergebnis auf dem LCD ausgeben:
	// Zur Position springen
	LcdJumpToPosition(line,
										column);
	// Das Arry mit dem Ergebnis elementeweise ausgeben
	// Dabei das Array mit dem letzten Element zuersst ausgeben, 
	// da die Binrzahl von rechts nach links gelesen wird
	for (i=size;i>0;i--)
	{
		LcdWriteChar(ASCII_CODE_FOR_ZERO+result[i-1]);
	}
}
//=== Function: LcdDrawPixel ======================================================================
///
/// @brief	Funktion zeichnet ein helles/dunkles einzelndes Pixel an eine frei whlbare Position
///
/// @param	uint16_t, x-Position; uint16_t, y-Position;  bool, hell/ dunkel
///
/// @return	void
//=================================================================================================
void LcdDrawPixel(uint16_t x, 
									uint16_t y, 
									bool color)
{
	// Adresse Grafikbyte
	uint16_t address = 0;
	// Zu setzendes Bit
	uint16_t bit = 0;
	// berprfen, ob die Pixelnummer zulssig ist (1 <= x <= 240, 1 <= y <= 64)
	// Wenn Pixelnummer zulssig: Pixel setzen/ lschen
	if ((x >= 1)
			&& (x <= PIXELS_OF_DISPLAY_X)
			&& (y >= 1)
			&& (y <= PIXELS_OF_DISPLAY_Y))
	{
		// Adresse des entsprechenden Grafik-RAM-Bytes ermitteln
		address = (  HOME_ADRESS_GRAPHIC
								+((y-1)*COLUMNS_OF_DISLPLAY)
								+((x-1)/FONT_SIZE)           );
		// Adresspointer auf errechnete Adresse setzen
		LcdSetAddressPointer(address);
		// Zu setzendes Bit ermitteln
		bit = (  FONT_SIZE
						 -1
						 -((x-1)%FONT_SIZE)  );
		// Je nach Parameter "color" das Bit setzen (weis) oder lschen (blau)
		if (color == WHITE)
		{
			// Bit setzen: 0xF8 + ensprechendes Bit
			LcdSendCommand(SET_BIT+bit);
		}
		if (color == BLUE)
		{
			// Bit Lschen: 0xF0 + ensprechendes Bit
			LcdSendCommand(DELETE_BIT+bit);
		}
	}
}
//=== Function: LcdDrawLineX ======================================================================
///
/// @brief	Funktion zeichnet eine helle/dunkle waagerechte Linie an eine frei whlbare Position
///
/// @param	uint16_t, Start; uint16_t, Ende; uint16_t y; uint16_t Strichstrke;  bool, hell/dunkel
///
/// @return	void
//=================================================================================================
int8_t calculateSignum(int16_t value)
{
	// Ergebnis
	int8_t result = 0;
	// Vorzeichen ermitteln
	if (value < 0)
	{
		result = -1;
	}
	if (value > 0)
	{
		result = 1;
	}
	// Ergbenis zurckgeben
	return result;
}
//=== Function: LcdDrawLine =======================================================================
///
/// @brief	Funktion zeichnet eine weie/ blaue Linie an eine frei whlbare Position
///					Der Algorithmus wurde bernommen von: http://de.wikipedia.org/wiki/Bresenham-Algorithmus
///
/// @param	uint16_t, Start; uint16_t, Ende; uint16_t y; bool, hell/dunkel
///
/// @return	void
//=================================================================================================
void LcdDrawLine(uint8_t x1,
								 uint8_t y1,
								 uint8_t x2,
								 uint8_t y2,
								 bool color)
{
	// Laufvariable
	uint16_t i = 0;
	// Variablen fr die Differenz der Strecke in X- und Y-Richtung
	int16_t deltaX = (x2-x1);
	int16_t deltaY = (y2-y1);
	// Variablen fr das Inkrement
	int16_t incrementX = calculateSignum(deltaX);
	int16_t incrementY = calculateSignum(deltaY);
	// Variablen fr Diagonalschritt
	int16_t diogonalStepX = incrementX;
	int16_t diogonalStepY = incrementY;
	// Variablen fr Parallelschritt
	int16_t parallelStepX = 0;
	int16_t parallelStepY = 0;
	// Variablen fr Fehlerschritt
	int16_t errorStepSlow = 0;
	int16_t errorStepFast = 0;
	// Fehlervariable
	int16_t error = 0;
	// Variablen fr die errechneten x- und y-Werte
	uint8_t x = x1;
	uint8_t y = y1;
	// Betrag der Strecken bilden, falls diese negativ ist/ sind
	if (deltaX < 0)
	{
		deltaX*=-1;
	}
	if (deltaY < 0)
	{
		deltaY*=-1;
	}
	// Berechnen welche Strecke (x oder y) grer ist
	if (deltaX > deltaY)
	{
		parallelStepX = incrementX;
		parallelStepY = 0;
		errorStepFast = deltaY;
		errorStepSlow = deltaX;
	}
	else
	{
		parallelStepX = 0;
		parallelStepY = incrementY;
		errorStepFast = deltaX;	
		errorStepSlow = deltaY;	
	}
	// Fehler bestimmen
	error = (errorStepSlow/2);
	// Erstes Pixel setzen
	LcdDrawPixel(x,y,color);
	// In einer Schleife die Pixel der Linie berechnen und ausgeben
	for (i=0;i<errorStepSlow;i++)
	{
		// Fehler neu berechnen
		error-=errorStepFast;
		// x- und y-Wert berechnen
		if (error < 0)
		{
			error+=errorStepSlow;
			x+=diogonalStepX;
			y+=diogonalStepY;
		}
		else
		{
			x+=parallelStepX;
			y+=parallelStepY;			
		}
		// Errechnetes Pixel setzen
		LcdDrawPixel(x,y,color);
	}
}
//=== Function: LcdDrawRectangle ==================================================================
///
/// @brief	Funktion zeichnet ein helles/dunkles Rechteck (Rahmen) an eine frei whlbare Position
///
/// @param	uint16_t, x1-Position; uint16_t, y1-Position; uint16_t, x2-Position;  
///         uint16_t, y2-Position; uint16_t Strichstrke; bool, hell/ dunkel
///
/// @return	void
//=================================================================================================
void LcdDrawRectangle(uint16_t x1, 
											uint16_t y1, 
											uint16_t x2, 
											uint16_t y2, 
											uint16_t thickness, 
											bool color)
{
	// Laufvariable
	uint8_t i = 0;
	// berprfen, ob die Koordinaten die richtige Reihenfolge besitzen
	// Nur wenn das Rechteck logisch korrekt ist, wird es gezeichnet
	if ((x1 < x2)
			&& (y1 < y2))
	{
		// Mit jeder Strichstrke (= ein Pixel) wchst das Rechteck nach innen
		for (i=0;i<thickness;i++)
		{	
			// Waagerechte Linien zeichnen mithilfe der effizienten Funktion "LcdSetGraphicArea"
			// Obere Linie
			LcdSetGraphicArea(x1,
												y1+i,
												x2,
												y1+i,
												color);
			// Untere Linie
			LcdSetGraphicArea(x1,
												y2-i,
												x2,
												y2-i,
												color);
			// Senkrechte Linien zeichen
			// Linke Linie 
			LcdDrawLine(x1+i,
									y1,
									x1+i,
									y2,
									color);
			// Rechte Linie
			LcdDrawLine(x2-i,
									y1,
									x2-i,
									y2,
									color);
		}		
	}
}
//=== Function: LcdSetGraphicArea =================================================================
///
/// @brief	Funktion setzt/ lscht den Inhalt des Grafik-RAM im bergebenen x- und y-Intervall.
///					Dabei werden, wenn mglich, ganze Bytes gesetzt/ gelscht. Der Parameter "color" gibt an,
///					ob das entsprechende Bit gesetzt (WHITE) oder gelscht (BLUE) werden soll.
///					In dieser Funktion ist von Bits die Rede, allerdings entspricht hier ein Bit ein Pixel.
///
/// @param	uint8_t x1, uint8_t y1, uint8_t y2, uint8_t x2, bool color
///
/// @return	void
//=================================================================================================
void LcdSetGraphicArea(uint8_t x1,
											 uint8_t y1,
											 uint8_t x2,
											 uint8_t y2,
											 bool color)
{
	// Funktion wird nur ausgefhrt, wenn die bergabeparameter korrekt sind
	if ((x1 <= x2)
			&&(y1 <= y2)
			&&(x1 > 0)
			&&(y2 > 0)
			&&(x2 <= PIXELS_OF_DISPLAY_X)
			&&(y2 <= PIXELS_OF_DISPLAY_Y))
	{
		// Laufvariablen
		int8_t i = 0;
		uint8_t j = 0;
		// Ganz zu setzende/ lschende Bytes im Intervall
		// Integer, weil der Wert je nach Intervallgrenzen auch negativ wird (bis max. -1)
		int16_t fullBytesBetweenFirstAndLastByte = 0;
		// Zu setzenden/ lschende Bits im ersten Byte des Intervalls
		uint8_t setBitsInFirstByte = 0;
		// Zu setzenden/ lschende Bits im letzten Byte des Intervalls
		uint8_t setBitsInLastByte = 0;
		// Erstes Bit des ersten Bytes im Intervall
		// Achtung: Das ist die Bitnummer, also (7 6) 5 4 3 2 1 0 | (7 6) 5 4 3 2 1 0 | (7 6) 5 4 3 2 1 0 | ...
		uint8_t firstBitOfFirstByte = FONT_SIZE-(x1+FONT_SIZE)%FONT_SIZE;
		// Letztes Bit des letzten Bytes im Intervall, auch hier: Das ist die Bitnummer
		uint8_t lastBitOfLastByte = FONT_SIZE-(x2+FONT_SIZE)%FONT_SIZE;
		// Wenn das erste Bit zugleich das letzte Bit des ersten Bytes ist, 
		// dann wird es manuell auf 0 gesetzt, da obige Operation nur die Zeichenbreite (6 oder 8) zurckgibt
		if (firstBitOfFirstByte == FONT_SIZE)
		{
			firstBitOfFirstByte = 0;
		}
		// Gleiche Berechnung wie oben, allerdings fr das letzte Bit des letzten Bytes
		if (lastBitOfLastByte == FONT_SIZE)
		{
			lastBitOfLastByte = 0;
		}
		// Gesetzte/ gelschte Bits des ersten Bytes in eine Variable kopieren
		for (i=firstBitOfFirstByte;i>=0;i--)
		{
			// Der erste Operand der if-Abfrage gibt die Position i an bezogen auf das ganze LCD, also die Pixelnummer
			// Alle Bits zwischen dem ersten und dem letzten Bit des Bytes, bzw. x2 (wenn x2 kleiner ist), werden gesetzt
			if (((((x1-1)/FONT_SIZE)*FONT_SIZE)+(FONT_SIZE-i)) <= x2)
			{
				setBitsInFirstByte |= (1<<i);
			}
		}
		// Gesetzte/ gelschte Bits des letzten Bytes in eine Variable kopieren
		for (i=(FONT_SIZE-1);i>=lastBitOfLastByte;i--)
		{
			// Der erste Operand der if-Abfrage gibt die Position i an bezogen auf das ganze LCD, also die Pixelnummer
			// Alle Bits zwischen dem ersten, bzw. x1 (wenn x1 grer ist), und dem letzten Bit des Bytes werden gesetzt
			if (((((x2-1)/FONT_SIZE)*FONT_SIZE)+(FONT_SIZE-i)) >= x1)
			{
				setBitsInLastByte |= (1<<i);
			}
		}		
		// Anzahl der komplett zu setzenden/ lschenden Bytes im Intervall berechnen
		fullBytesBetweenFirstAndLastByte = (((x2-1)/FONT_SIZE)-((x1-1)/FONT_SIZE)-1);
		// Zeilenweise ganze Bytes setzen/ lschen
		for (i=y1;i<=y2;i++)
		{
			// Adresspointer setzten
			LcdSetAddressPointer(	HOME_ADRESS_GRAPHIC
													 +((x1-1)/FONT_SIZE)
													 +((i-1)*COLUMNS_OF_DISLPLAY)	);
			// Erstes Byte setzen/ lschen
			// Byte setzen (wei)
			if (color == WHITE)
			{
				LcdWriteData(setBitsInFirstByte);
			}
			// Byte lschen (blau)
			if (color == BLUE)
			{
				// Byte invertieren
				LcdWriteData((setBitsInFirstByte ^= 0xFF));
				// Byte zurck invertieren, damit beim nchsten Durchgang der selbe Wert bertragen wird
				setBitsInFirstByte ^= 0xFF;
			}	
			// Volle Bytes zwischen den ersten und letzten Byte setzen/ lschen
			for (j=0;j<fullBytesBetweenFirstAndLastByte;j++)
			{
				// Byte setzen (wei)
				if (color == WHITE)
				{
					LcdWriteData(setAllBitsOfFont);
				}
				// Byte lschen (blau)
				if (color == BLUE)
				{
					// Byte invertieren
					LcdWriteData((setAllBitsOfFont ^= 0xFF));
					// Byte zurck invertieren, damit beim nchsten Durchgang der selbe Wert bertragen wird
					setAllBitsOfFont ^= 0xFF;
				}
			}
			// Letztes Byte nur setzen/ lschen, wenn dieses nicht gleichzeitig auch das erste Byte ist
			if (((x1-1)/FONT_SIZE) != ((x2-1)/FONT_SIZE))
			{
				// Byte setzen (wei)
				if (color == WHITE)
				{
					LcdWriteData(setBitsInLastByte);
				}
				// Byte lschen (blau)
				if (color == BLUE)
				{
					// Byte invertieren
					LcdWriteData((setBitsInLastByte ^= 0xFF));
					// Byte zurck invertieren, damit beim nchsten Durchgang der selbe Wert bertragen wird
					setBitsInLastByte ^= 0xFF;
				}
			}
		}			
	}
	// bergabeparameter sind fehlerhaft
	else
	{
		// Fehlermeldung ausgeben
		LcdWriteString("Falsche Parameter",3,1);
	}
}
//=== Function: LcdDrawCircle =====================================================================
///
/// @brief	Funktion zeichnet einen weien/ blauen Kreis an eine frei whlbare Position
///					Der Algorithmus wurde bernommen von: http://de.wikipedia.org/wiki/Bresenham-Algorithmus
///
/// @param	uint8_t xCenter, uint8_t yCenter, uint8_t radius, bool hell/dunkel
///
/// @return	void
//=================================================================================================
void LcdDrawCircle(uint8_t xCenter, 
									 uint8_t yCenter, 
									 uint8_t radius,
									 bool color)
{
	// Variablen
  int16_t error =	(1-radius);
	int16_t ddF_x = 0;
	int16_t ddF_y = (-2*radius);
	int16_t x =			0;
	int16_t y =			radius;
	// Die vier Punkte zeichnen, welche direkt auf der x- bzw. y-Achse liegen
	LcdDrawPixel(xCenter, 
							 (yCenter+radius),
							 color);
	LcdDrawPixel(xCenter, 
							 (yCenter-radius),
							 color);
	LcdDrawPixel((xCenter+radius),
							 yCenter,
							 color);
	LcdDrawPixel((xCenter-radius),
							 yCenter,
							 color);
	// Restlichen Punkte zeichnen
	while(x < y)
	
	{
		// Fehler berechnen
		if(error >= 0)
		{
			y--;
			ddF_y += 2;
			error += ddF_y;
		}
		x++;
		ddF_x += 2;
		error += ddF_x + 1;
		// Pixel zeichnen
		LcdDrawPixel(xCenter + x, yCenter + y,color);
		LcdDrawPixel(xCenter - x, yCenter + y,color);
		LcdDrawPixel(xCenter + x, yCenter - y,color);
		LcdDrawPixel(xCenter - x, yCenter - y,color);
		LcdDrawPixel(xCenter + y, yCenter + x,color);
		LcdDrawPixel(xCenter - y, yCenter + x,color);
		LcdDrawPixel(xCenter + y, yCenter - x,color);
		LcdDrawPixel(xCenter - y, yCenter - x,color);
	}
}
//=== Function: lcdWriteUseSpecificSign ===========================================================
///
/// @brief	Funktion gibt ein benutzerdefiniertes Zeichen an eine Textposition (Zeile, Spalte) aus
///
/// @param	uint8_t sign, uint8_t line, uint8_t column
///
/// @return	void
//=================================================================================================
void lcdWriteUseSpecificSign(uint8_t sign,
														 uint8_t line,
														 uint8_t column)
{
	// Laufvariable
	uint8_t i = 0;
	// Pointer fr die Elemente der Bitmaps
	uint8_t *elementOfBitmap = 0;
	// Je nach bertragenden Zeichen, wird der Pointer auf das 1. Element der entsprechenden Bitmap gesetzt
	switch (sign)
	{
		case DEGREE_SIGN:
				elementOfBitmap = &bitmapDegreeSign[0];
				break;
	}
	// Adresspoiter auf die Position setzen, an der geschrieben werden soll
	LcdSetAddressPointer(	HOME_ADRESS_GRAPHIC																		// Startadresse Grafikspeicher
											 +(COLUMNS_OF_DISLPLAY*(line-1)*HEIGHT_SPECIFIC_SIGN)		// Zeilenoffset
											 +(column-1)																				 );	// Spaltenoffset
	// Zeilenweise das Zeichen (Bitmap) ausgeben
	for (i=0;i<HEIGHT_SPECIFIC_SIGN;i++)
	{
		// Element der Bitmap setzen
		LcdWriteData(*elementOfBitmap);
		// Pointer um 1 erhhen/ auf das nchste Element zeigen lassen
		elementOfBitmap++;
		// Adresspoiter auf die neue Position setzen
		LcdSetAddressPointer(	HOME_ADRESS_GRAPHIC																		// Startadresse Grafikspeicher
												 +(COLUMNS_OF_DISLPLAY*(line-1)*HEIGHT_SPECIFIC_SIGN)		// Zeilen-Offset
											   +(column-1)																						// Spalten-Offset
												 +(COLUMNS_OF_DISLPLAY*(i+1))												 );	// Zeichenhhen-Offset
	}
}	