//=================================================================================================
/// \file:		Temperatursensor.c
///
/// \date:		01.08.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include "Temperatursensor.h"
#include "TwoWireInterface.h"
#include "LCD240X064.h"
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Temperaturdifferenz durch Eigenerwrmung des Sensor
// Auf S. 35 des Datenblattes steht die Formel zur Berechnung der Eigenerwrmung
// Hier ist eine grobe Schtzung unter Vernachlssigung von Verlusten durch den SDA- und Alarm-Pin
volatile double temperaturOfSelfHeating = 0.25;
//-------------------------------------------------------------------------------------------------
// Functions
//-------------------------------------------------------------------------------------------------
//=== Function: MCP9808SetRegisterPointer =========================================================
///
/// @brief	Funktion setzt den Registerpointer auf ein betimmtes Register
///
/// @param	uint8_t registerNumber
///
/// @return	void
//=================================================================================================
void MCP9808SetRegisterPointer(uint8_t registerNumber)
{
	// Sensor mit Adresse und Schreibbefehl ansprechen und anschlieend ein Byte bertragen
	TwiTransmitSingleByte(SLAVE_ADDRESS_MCP9808,
												registerNumber);
}
//=== Function: MCP9808SetResolutionRegister ======================================================
///
/// @brief	Funktion setzt die Auflsung der Temperaturmessung
///
/// @param	uint8_t resolution
///
/// @return	void
//=================================================================================================
void MCP9808SetResolutionRegister(uint8_t resolution)
{
	// Array fr die zu sendenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {REGISTER_TEMPERATURE_RESOLUTION,
																			 resolution};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Daten senden
	TwiTransmitByteArray(SLAVE_ADDRESS_MCP9808,
											 firstElement,
											 TWO_DATA_BYTES);																						
}
//=== Function: MCP9808SetConfigurationRegister ===================================================
///
/// @brief	Funktion setzt das Konfigurationsregister
///
/// @param	uint8_t registerNumber
///
/// @return	void
//=================================================================================================
void MCP9808SetConfigurationRegister(void)
{
	// Array fr die zu sendenen Daten anlegen
	uint8_t dataBytes[THREE_DATA_BYTES] = {0,0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Datenbytes setzen
	// Konfigurationsregister
	dataBytes[0] = REGISTER_CONFIGURATION;
	// Oberes Datenbyte (keine Hysterese, active-Modus)
	dataBytes[1] = 0x00;
	// Unteres Datenbyte (Temp.-Grenzen nicht gelockt, Alarm aktiv, Compare-Modus, low-active)
	dataBytes[2] = 0x18;
	// Daten senden
	TwiTransmitByteArray(SLAVE_ADDRESS_MCP9808,
											 firstElement,
											 THREE_DATA_BYTES);	
}
//=== Function: MCP9808SetTemperatureLimitAlert ===================================================
///
/// @brief	Funktion setzt die Temperaturgrenzen bei denen der Alarmpin ausgelst wird
///					Es werden zwischen der oberen, der unteren und der kritischen Temperaturgrenze unteschieden
///					Das genaue Verhalten des Alarmpins ist dem Datenblat auf Seite 32 zu entnehmen
///
/// @param double temperatureLimit, uint8_t temperatureRegister
///
/// @return	void
//=================================================================================================
void MCP9808SetTemperatureLimitAlert(double temperatureLimit,
																		 uint8_t temperatureRegister)
{
	// Array fr die zu sendenen Daten anlegen
	uint8_t dataBytes[THREE_DATA_BYTES] = {0,0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Temperaturgrenzen berprfen (-40C <= T <= 125C)
	// Wenn Temperaturgrenze zu gering
	if (temperatureLimit < MINIMUM_TEMPERATURE)
	{
		// Temperaturgrenze auf die Untergrenze setzen
		temperatureLimit = MINIMUM_TEMPERATURE;
	}
	// Wenn Temperaturgrenze zu hoch
	if (temperatureLimit > MAXIMUM_TEMPERATURE)
	{
		// Temperaturgrenze auf die Obergrenze setzen
		temperatureLimit = MAXIMUM_TEMPERATURE;
	}
	// Temperaturgrenze unter Bercksichtigung der Sensor-Eigenerwrmung korrigieren 
	temperatureLimit += temperaturOfSelfHeating;
	// Oberes und Unteres Byte berechnen, Formel hierzu ist dem Datenblatt S. 22 zu entnehmen
	// int8_t statt uint8_t weil "temperatureLimit" vorzeichenbehaftet ist
	uint8_t upperByte =	((int8_t)temperatureLimit >> 4);
	uint8_t lowerByte = ((int8_t)temperatureLimit << 4);	
	// Betrag der Nachkommastelle des Temperaturlimits berechnen
	// Betrag bilden
	if (temperatureLimit < 0)
	{
		temperatureLimit *= (-1);
	}
	// Nachkommastelle berechnen
	temperatureLimit = (temperatureLimit - (uint8_t)temperatureLimit);
	// Je nach Gre der Nachkommastelle die Bits 2 und 3 des "lowerBytes" setzen
	// Der genaue Zusammenhanng zwischen den Bits und der Temperaturgrenze ist dem Datenblatt S. 22 zu entnehmen
	// Nachkommastellen knnen nur in 0,25C-Schritten aufgelst werden
	if ((temperatureLimit > 0) 
		 && (temperatureLimit <= 0.35))
	{
		// Bit 2 setzen
		lowerByte |= (1<<2);
	}	
	if ((temperatureLimit > 0.35) 
		 && (temperatureLimit <= 0.6))
	{
		// Bit 3 setzen
		lowerByte |= (1<<3);
	}
	if ((temperatureLimit > 0.6) 
		 && (temperatureLimit < 1))
	{
		// Bit 2 und 3 setzen
		lowerByte |= ((1<<3)|(1<<2));
	}
	// Zu sendenen Bytestrom initialisieren
	// Registeradresse der gewhlten Temperaturgrenze (obere, untere oder kritische Temperatur)
	dataBytes[0] = temperatureRegister;
	// Oberes Datenbyte
	dataBytes[1] = upperByte;
	// Unteres Datenbyte
	dataBytes[2] = lowerByte;															
	// Daten senden
	TwiTransmitByteArray(SLAVE_ADDRESS_MCP9808,
											 firstElement,
											 THREE_DATA_BYTES);
}
//=== Function: MCP9808ReadTemperatureLimitAlert ==================================================
///
/// @brief	Funktion liest die gesetzten Temperaturgrenzen fr den Alarm-Pin aus
///
/// @param	void
///
/// @return	double
//=================================================================================================
double MCP9808ReadTemperatureLimitAlert(uint8_t temperatureRegister)
{
	// Rckgabewert
	double temperature = 0;
	// Array fr die empfangenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Registerpointer auf das entsprechende Temperatur-Register (upper, lower oder critical) setzen
	MCP9808SetRegisterPointer(temperatureRegister);	
	// Temperatur-Register auslesen
	TwiReceiveByteArray(SLAVE_ADDRESS_MCP9808,
											firstElement,
											TWO_DATA_BYTES);	
	// Temperatur berechnen
	// Vorkomma-Anteil
	temperature =	(int8_t)((dataBytes[0] << 4)+(dataBytes[1] >> 4));
	// Nachkommanteil berechnen
	// Der Zusammenhanng zwischen den Bits und des Nachkomma-Anteils ist dem Datenblatt S. 22 zu entnehmen
	switch (dataBytes[1] & 0x0C)
	{
		// Wenn Bit 2 gesetzt ist: Nachkommastelle 0,25C
		case 0x04:
				temperature += 0.25;
				break;
		// Wenn Bit 3 gesetzt ist: Nachkommastelle 0,5C
		case 0x08:
				temperature += 0.5;
				break;
		// Wenn Bit 2 und 3 gesetzt sind: Nachkommastelle 0,75C
		case 0x0C:
				temperature += 0.75;
				break;
	}
	// Temperaturgrenze unter Bercksichtigung der Sensor-Eigenerwrmung korrigieren
	temperature -= temperaturOfSelfHeating;
	// Errechnete Temperatur zurckgeben
	return temperature;
}
//=== Function: MCP9808Init =======================================================================
///
/// @brief	Funktion setzt das Konfigurationsregister und die Temperaturlimits fr den Alarmpin
///
/// @param	void
///
/// @return	void
//=================================================================================================
void MCP9808Init(void)
{
	// Konfigurationsregister setzen
	MCP9808SetConfigurationRegister();
	// Oberes Temperaturlimit setzen
	MCP9808SetTemperatureLimitAlert(30,
																	REGISTER_UPPER_TEMPERATURE_ALARM_LIMIT);
	// Unteres Temperaturlimit setzen
	MCP9808SetTemperatureLimitAlert(-10,
																	REGISTER_LOWER_TEMPERATURE_ALARM_LIMIT);
	// Kritisches Temperaturlimit setzen
	MCP9808SetTemperatureLimitAlert(50,
																	REGISTER_CRITICAL_TEMPERATURE_LIMIT);
	// Temperaturauflsung setzen
	MCP9808SetResolutionRegister(RESOLUTION_0_POINT_0625_C);
}
//=== Function: MCP9808ReadAmbientTemperature =====================================================
///
/// @brief	Funktion liest die Temperatur des Sensors aus
///
/// @param	void
///
/// @return	double
//=================================================================================================
double MCP9808ReadAmbientTemperature(void)
{
	// Rckgabewert
	double temperature = 0;
	// Array fr die empfangenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Registerpointer auf das Temperatur-Register setzen
	MCP9808SetRegisterPointer(REGISTER_AMBIENT_TEMPERATURE);	
	// Temperatur-Register auslesen
	TwiReceiveByteArray(SLAVE_ADDRESS_MCP9808,
											firstElement,
											TWO_DATA_BYTES);
	// Je nach dem ob die Temperatur grer gleich 0C oder kleiner 0C ist, wird die Temperatur anders berechnet
	// Das 4. Bit des oberen Bytes gibt das Vorzeichen der Temperatur an
	// Die Temperatur ist negativ, wenn das Bit gesetzt ist
	if ((dataBytes[0] & 0x10) == 0x10)
	{
		// Die Berechnungsformel fr die negative Temperatur steht im Datenblatt des Sensors
		temperature = -(256-(((dataBytes[0] & 0x0F)*16)+((double)dataBytes[1]/16)));
	}
	// Temperatur ist positiv, wenn das Bit nicht gesetzt ist
	else
	{
		// Die Berechnungsformel fr die positive Temperatur steht im Datenblatt des Sensors
		temperature = (((dataBytes[0] & 0x0F)*16)+((double)dataBytes[1]/16));
	}
	// Temperaturgrenze unter Bercksichtigung der Sensor-Eigenerwrmung korrigieren
	temperature -= temperaturOfSelfHeating;
	// Errechnete Temperatur zurckgeben
	return temperature;
}
//=== Function: MCP9808ReadManufacturerID =========================================================
///
/// @brief	Funktion liest die Herstellernummer des Sensors aus
///
/// @param	void
///
/// @return	uint16_t
//=================================================================================================
uint16_t MCP9808ReadManufacturerID(void)
{
	// Rckgabewert
	uint16_t result = 0;
	// Array fr die empfangenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Registerpointer auf das Herstellernummer-Register setzen
	MCP9808SetRegisterPointer(REGISTER_MANUFACTURER_ID);
	// Nummer auslesen 
	TwiReceiveByteArray(SLAVE_ADDRESS_MCP9808,
											firstElement,
											TWO_DATA_BYTES);
	// Wert berrechen (erstes Byte um 8 Stellen nach links shiften, letztes Byte drauf addieren)
	result = ((dataBytes[0] << 8)+dataBytes[1]);
	// Ergebnis zurckgeben
	return result;
}
//=== Function: MCP9808ReadDeviceID ===============================================================
///
/// @brief	Funktion liest die Bauteil-Identifikationsnummer des Sensors aus
///
/// @param	void
///
/// @return	uint8_t
//=================================================================================================
uint8_t MCP9808ReadDeviceID(void)
{
	// Array fr die empfangenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Registerpointer auf das Bauteil-ID/ Revisions-Register setzen
	MCP9808SetRegisterPointer(REGISTER_DEVICE_ID);
	// Nummer auslesen 
	TwiReceiveByteArray(SLAVE_ADDRESS_MCP9808,
											firstElement,
											TWO_DATA_BYTES);
	// Das Upper-Byte ist die Bauteil-ID
	return dataBytes[0];
}
//=== Function: MCP9808ReadDeviceRevision =========================================================
///
/// @brief	Funktion liest die Revisionsnummer des Sensors aus
///
/// @param	void
///
/// @return	uint8_t
//=================================================================================================
uint8_t MCP9808ReadDeviceRevision(void)
{
	// Array fr die empfangenen Daten anlegen
	uint8_t dataBytes[TWO_DATA_BYTES] = {0,0};
	// Pointer anlegen und auf das erste Element des Datenarrys zeigen lassen
	uint8_t *firstElement = &dataBytes[0];
	// Registerpointer auf das Bauteil-ID/ Revisions-Register setzen
	MCP9808SetRegisterPointer(REGISTER_DEVICE_ID);
	// Nummer auslesen 
	TwiReceiveByteArray(SLAVE_ADDRESS_MCP9808,
											firstElement,
											TWO_DATA_BYTES);
	// Das Lower-Byte ist die Bauteil-ID
	return dataBytes[1];
}
//=== Function: MCP9808WriteAmbientTemperatureOnLcd ===============================================
///
/// @brief	Funktion liest die Umgebungstemperatur des Sensors auf ein LCD aus
///
/// @param	double temperature, uint8_t line, uint8_t column
///
/// @return	void
//=================================================================================================
void MCP9808WriteAmbientTemperatureOnLcd(double temperature,
																				 uint8_t line,
																				 uint8_t column)
{
	// Temperaturwert ausgeben (zwei Vorkomma-, eine Nachkommastelle)
	LcdWriteDouble(temperature,
								 2,
								 1,
								 line,
								 column);
	// Gradzeichen ausgeben
	lcdWriteUseSpecificSign(DEGREE_SIGN,
													line,
													column+5);
	// Einheit ausgeben (Celsius)
	LcdWriteString("C",
								 line,
								 column+6);
}