//=================================================================================================
/// \file:		TwoWireInterface.c
///
/// \date:		26.03.2014
///
/// \version: 1.0
///
/// \author:  Daniel 
//=================================================================================================
//-------------------------------------------------------------------------------------------------
// Includes
//-------------------------------------------------------------------------------------------------
#include "TwoWireInterface.h"
#include "LCD240X064.h"
//-------------------------------------------------------------------------------------------------
// Variables
//-------------------------------------------------------------------------------------------------
// Fehlerspeicher
volatile uint8_t firstError = ERROR_NONE;
//-------------------------------------------------------------------------------------------------
// Functions
//-------------------------------------------------------------------------------------------------
//=== Function: TwiInit ===========================================================================
///
/// @brief	Funktion initialisiert das Two-Wire-Interface (setzen der Taktfrequenz)
///
/// @param	uint16_t sclFrequencykHz
///
/// @return	void
//=================================================================================================
void TwiInit(uint16_t sclFrequency)
{
	// Bitrate setzen 
	// Die Formel zur Berechnung steht auf Seite 207 des Datenblattes des ATmega644
	TWBR = (uint8_t)((F_CPU/(8*sclFrequency))-2);
}
//=== Function: TwiStartCondition =================================================================
///
/// @brief	Funktion erzeugt ein Start-Signal
///
/// @param	void
///
/// @return	void
//=================================================================================================
void TwiStartCondition(void)
{
	// TWICR-Register initialisieren
	// Interruptflag lschen (durch schreiben einer 1), Start-Bit und TWI-Enable-Bit setzen 
	// Alle restlichen Bits sind 0
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	// Warten bis die Startkondition gesendet wurde 
	TwiWaitForInterruptFlag();
	// Falls ein Fehler aufgetreten ist (und der Fehlerspeicher leer ist) wird dieser in einem Fehlerspeicher abgelegt
	// Zur Abfrage des Staus mssen die Bits 0 bis 2 maskiert werden (diese sind fr etwas Anderes zustndig)
	// Die Codes des Statusregisters sind im Datenblatt hinterlegt, Dabei auf die Betriebsart achten
	if (((TWSR & 0xF8) != START_CONDITION_SENDED)
		  && (firstError == ERROR_NONE))
	{
		// Fehlerzuordnung
		firstError = ERROR_START_CONDITION;
	}
}
//=== Function: TwiRepeatedStartCondition =========================================================
///
/// @brief	Funktion erzeugt ein wiederholtes Start-Signal
///
/// @param	void
///
/// @return	void
//=================================================================================================
void TwiRepeatedStartCondition(void)
{
	// Interruptflag lschen (durch schreiben einer 1), Start-Bit und TWI-Enable-Bit setzen
	// Alle restlichen Bits sind 0
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);	
	// Warten bis die wiederholte Startkondition gesendet wurde 
	TwiWaitForInterruptFlag();
	// Falls ein Fehler aufgetreten ist (und der Fehlerspeicher leer ist) wird dieser in einem Fehlerspeicher abgelegt
	// Zur Abfrage des Staus mssen die Bits 0 bis 2 maskiert werden (diese sind fr etwas Anderes zustndig)
	// Die Codes des Statusregisters sind im Datenblatt hinterlegt, Dabei auf die Betriebsart achten
	if (((TWSR & 0xF8) != REPEATED_START_CONDITION_SENDED)
		  && (firstError == ERROR_NONE))
	{
		// Fehlerzuordnung
		firstError = ERROR_REPEATED_START_CONDITION;
	}
}
//=== Function: TwiStopCondition ==================================================================
///
/// @brief	Funktion erzeugt ein Stop-Signal
///
/// @param	void
///
/// @return	void
//=================================================================================================
void TwiStopCondition(void)
{
	// Interruptflag lschen und Stop-Bit setzen
	// Alle restlichen Bits sind 0
	TWCR = (1<<TWINT)|(1<<TWSTO);	
}
//=== Function: TwiWaitForInterruptFlag ===========================================================
///
/// @brief	Funktion wartet (verweilt) solange, bis das Interruptflag des TWI gesetzt wurde
///         Wird dabei eine bestimmete Zeit berschritten, wird das Warten abgebrochen 
///
/// @param	void
///
/// @return	void
//=================================================================================================
void TwiWaitForInterruptFlag(void)
{
	// Schleifenzhlvariable
	uint16_t counter = 0;
	// Warteschleife
	// Bei jedem Schleifendurchgang wird 10s gewartet und "counter" um 1 erhht
	// Sobald TWINT gesetzt ist oder "counter" den Wert 5000 (entspricht 0,05 Sekunden) berschreitet, 
	// wird die Warteschleife beendet
	// TIME_OUT dient dazu die Schleife abzubrechen, wenn keine bertragung stattfindet
	while (!((TWCR & (1<<TWINT))
					|| (counter >= TIME_OUT)))
	{
		// Kurz warten
		_delay_us(10);
		// Counter erhhen
		counter++;	
	} 
}
//=== Function: TwiAddressSlave ===================================================================
///
/// @brief	Funktion sendet das erste Byte eines Datentransfers (Slaveadresse und Operationsbefehl (R/W))
///
/// @param	uint8_t slaveAddress, bool operation
///
/// @return	void
//=================================================================================================
void TwiAddressSlave(uint8_t slaveAddress,
										 bool operation)
{	
	// Zu sendendes Byte
	uint8_t addressAndOperation = 0;
	// STARTKONDITION SENDEN
	// Start-Signal erzeugen
	TwiStartCondition(); 
	// ZU BERTRAGENDES BYTE VORBEREITEN
	// Adresse + Operation (lesen oder schreiben) in einem Byte speichern
	// Durch Shiften wird eine 0 hinten angehngt, das ist der Schreibbefehl
	addressAndOperation = (slaveAddress << 1);
	// Wenn gelesen werden soll, wird das letzte Bit gesetzt
	if (operation == READ_DATA)
	{
		addressAndOperation |= (1<<0);
	}	
	// BYTE AN DEN SLAVE BERTRAGEN
	// Slave-Adresse in das Datenregister schreiben
	TWDR = addressAndOperation;
	// Interruptflag lschen und TWI-Enable-Bit setzen
	TWCR = (1<<TWINT)|(1<<TWEN);
	// Warten bis die Slave-Adresse (inklusive dem Schreibbefehl) gesendet wurde und ein ACK/ NACK empfangen wurde
	// Nur wenn beides erfolgt ist, wird TWINT gesetzt
	TwiWaitForInterruptFlag();
	// BERTRAGUNGSSTATUS ADRESSE UND ACK PRFEN
	// Schreibbefehl
	if (operation == WRITE_DATA)
	{
		// Prfen ob die Slave-Adresse und der Befehl (write) fehlerfrei gesendet und ein ACK vom Slave empfangen wurde
		if (((TWSR & 0xF8) != SLAVE_ADDRESS_WRITE_SENDED_ACK_RECEIVED)
			 && (firstError == ERROR_NONE))
		{
			// Fehlerzuordnung
			firstError = ERROR_SLAVE_ADDRESS_WRITE;
		} 		
	}
	// Lesebefehl
	if (operation == READ_DATA)
	{
		// Prfen ob die Slave-Adresse und der Befehl (read) fehlerfrei gesendet und ein ACK vom Slave empfangen wurde
		if (((TWSR & 0xF8) != SLAVE_ADDRESS_READ_SENDED_ACK_RECEIVED)
			 && (firstError == ERROR_NONE))
		{
			// Fehlerzuordnung
			firstError = ERROR_SLAVE_ADDRESS_READ;
		}
	}
}
//=== Function: TwiTransmitSingleByte =============================================================
///
/// @brief	Funktion sendet ein Datenbyte an einen Slave (Master-Transmitter-Mode)
///
/// @param	uint8_t slaveAddress, uint8_t dataByte
///
/// @return	void
//=================================================================================================
void TwiTransmitSingleByte(uint8_t slaveAddress,
													 uint8_t dataByte)
{
	// Adresse des Slaves und Schreibbefehl senden
	TwiAddressSlave(slaveAddress,
									WRITE_DATA);
	// Datenbyte senden
	// Daten in das Datenregister schreiben
	TWDR = dataByte;
	// Interruptflag lschen und TWI-Enable-Bit setzen
	TWCR = (1<<TWINT)|(1<<TWEN);
	// Warten bis das Datenbyte an den Slave gesendet wurde und ein ACK/ NACK empfangen wurde
	// Nur wenn beides erfolgt ist, wird TWINT gesetzt
	TwiWaitForInterruptFlag();
	// Prfen ob das Datenbyte fehlerfrei gesendet und ein ACK vom Slave empfangen wurde
	if (((TWSR & 0xF8) != DATA_BYTE_SENDED_ACK_RECEIVED)
			&& (firstError == ERROR_NONE))
	{
		// Fehlerzuordnung
		firstError = ERROR_DATA_WRITE;
	}	
	// Stop-Signal erzeugen
	TwiStopCondition();
}
//=== Function: TwiReceivedSingleByte =============================================================
///
/// @brief	Funktion liest ein Datenbyte von einem Slave (Master-Receiver-Mode)
///
/// @param	uint8_t slaveAddress
///
/// @return	uint8_t data
//=================================================================================================
uint8_t TwiReceiveSingleByte(uint8_t slaveAddress)
{
	// Rckgabewert
	uint8_t receivedByte = 0;
	// Adresse des Slaves und Lesebefehl senden
	TwiAddressSlave(slaveAddress,
									READ_DATA); 
	// Datenbyte vom Slave auf den Bus legen lassen
	// Interruptflag lschen, TWI-Enable-Bit und NACK-Bit setzen (keine weiteren Bytes mehr empfangen)
	TWCR = (1<<TWINT)|(1<<TWEN);
	// Warten bis das Datenbyte des Slaves empfangen wurde 
	TwiWaitForInterruptFlag();
	// Prfen ob das Datenbyte fehlerfrei empfangen und ein NACK an den Slave gesendet wurde
	if (((TWSR & 0xF8) != DATA_BYTE_RECEIVED_NACK_SENDED)
		  && (firstError == ERROR_NONE))
	{
		// Fehlerzuordnung
		firstError = ERROR_DATA_READ;
	}
	// Datenregister auslesen
	receivedByte = TWDR;
	// Stop-Signal erzeugen
	TwiStopCondition();
	// Ergebnis zurckgeben
	return receivedByte;
}
//=== Function: TwiTransmitByteArray ==============================================================
///
/// @brief	Funktion sendet mehrere Datenbytes an einen Slave (Master-Transmitter-Mode)
///					Die bergabe der Datenbytes erfolg ber einen Pointer, der auf ein Feld zeigt,
///					welches im Hauptprogramm angelegt wurde
///
/// @param	uint8_t slaveAddress, uint8_t *firstElementOfDataByteArray, uint8_t numberOfDataBytes
///
/// @return	void
//=================================================================================================
void TwiTransmitByteArray(uint8_t slaveAddress,
													uint8_t *firstElementOfDataByteArray,
													uint8_t numberOfDataBytes)
{
	// Schleifenkopfvariable
	uint8_t i = 0;
	// Adresse des Slaves und Schreibbefehl senden
	TwiAddressSlave(slaveAddress,
									WRITE_DATA);
	// Beide Datenbytes in einer Schleife an den Slave senden
	for (i=0;i<numberOfDataBytes;i++)
	{
		// Datenbyte in das Datenregister schreiben
		// Das Datenbyte ist der Wert der an der Adresse gespeichert ist, 
		// auf  welche der Pointer zeigt 
		TWDR = *firstElementOfDataByteArray;
		// Interruptflag lschen und TWI-Enable-Bit setzen
		TWCR = (1<<TWINT)|(1<<TWEN);
		// Warten bis das Datenbyte an den Slave gesendet wurde und ein ACK/ NACK empfangen wurde
		// Nur wenn beides erfolgt ist, wird TWINT gesetzt
		TwiWaitForInterruptFlag();
		// Prfen ob das Datenbyte fehlerfrei gesendet und ein ACK vom Slave empfangen wurde
		if (((TWSR & 0xF8) != DATA_BYTE_SENDED_ACK_RECEIVED)
				&& (firstError == ERROR_NONE))
		{
			// Fehlerzuordnung
			firstError = ERROR_DATA_WRITE;
		}	
		// Der Pointer wird um einen Wert nach oben gezhlt, 
		// damit er beim nchsten Durchgang auf das nchste zu sendene Element zeigt
		// Pointer wird solange inkrementiert, bis das letzte Element gesendet wurde, d.h. bis nach dem vorletzten Byte 
		if (i < (numberOfDataBytes-1))
		{
			firstElementOfDataByteArray++;
		}
	}
	// Stop-Signal erzeugen
	TwiStopCondition(); 
}
//=== Function: TwiReceiveByteArray ==============================================================
///
/// @brief	Funktion empfngt mehrere Datenbytes von einem Slave (Master-Receiver-Mode)
///					Die Speichern der Datenbytes erfolg ber einen Pointer, der auf ein Feld zeigt,
///					welches im Hauptprogramm angelegt wurde
///
/// @param	uint8_t slaveAddress, uint8_t *firstElementOfDataByteArray, uint8_t numberOfDataBytes
///
/// @return	void
//=================================================================================================
void TwiReceiveByteArray(uint8_t slaveAddress,
												 uint8_t *firstElementOfDataByteArray,
												 uint8_t numberOfDataBytes)
{
	// Laufvariable
	uint8_t i = 0;
	// Adresse des Slaves mit Lesebefehl senden
	TwiAddressSlave(slaveAddress,
									READ_DATA);
	// In einer Schleife die Bytes einzelnend empfangen
	// Das letzte Byte wird gesondert empfangen, weil danach ein NACK zurckgegeben wird
	for (i=0;i<(numberOfDataBytes-1);i++)
	{
		// Datenbyte vom Slave auf den Bus legen lassen
		// Interruptflag lschen, TWI-Enable-Bit und ACK-Bit setzen (um danach ein weiteres Bit zu empfangen)
		TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWEA);
		// Warten bis das Datenbyte von des Slaves empfangen wurde
		TwiWaitForInterruptFlag();
		// Prfen ob das Datenbyte fehlerfrei empfangen und ein ACK an den Slave gesendet wurde
		if (((TWSR & 0xF8) != DATA_BYTE_RECEIVED_ACK_SENDED)
				&& (firstError == ERROR_NONE))
		{
			// Fehlerzuordnung
			firstError = ERROR_DATA_READ;
		}
		// Datenregister auslesen und in dem globalen Array speichern
		*firstElementOfDataByteArray = TWDR;
		// Der Pointer wird um einen Wert nach oben gezhlt,
		// damit er beim nchsten Durchgang auf das nchste zu sendene Element zeigt
		firstElementOfDataByteArray++;
	}
	// Letztes Datenbyte vom Slave auf den Bus legen lassen
	// Interruptflag lschen, TWI-Enable-Bit und NACK-Bit setzen (keine weiteren Bytes mehr empfangen)
	TWCR = (1<<TWINT)|(1<<TWEN);
	// Warten bis das Datenbyte von dem Slave empfangen wurde 
	TwiWaitForInterruptFlag();
	// Prfen ob das Datenbyte fehlerfrei empfangen und ein NACK an den Slave gesendet wurde
	if (((TWSR & 0xF8) != DATA_BYTE_RECEIVED_NACK_SENDED)
		  && (firstError == ERROR_NONE))
	{
		// Fehlerzuordnung
		firstError = ERROR_DATA_READ;
	}
	// Datenregister auslesen und in dem globalen Array speichern
	*firstElementOfDataByteArray = TWDR;
	// Stop-Signal erzeugen
	TwiStopCondition();
}
//=== Function: TwiFailureHandling ================================================================
///
/// @brief	Funktion gibt den den ersten Fehler der ersten bertragung auf dem LCD aus
///					Der Fehlerspeicher muss manuelle gelscht werden (Hauptprogramm)
///
/// @param	void
///
/// @return	void
//=================================================================================================
void TwiFailureHandling(void)
{
	// Fehlerfeld ausgeben
	// Leerzeichen nach dem Doppelpunkt dienen zum graphischen Lschen
	LcdWriteString("Error:              ",
								 ERROR_OUTPUT_FIELD_LINE,
								 ERROR_OUTPUT_FIELD_COLUMN);
	// Fehlerart ausgeben:
	switch (firstError)
	{
		// Kein Fehler
		case ERROR_NONE:
				LcdWriteString("None",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler beim Erzeugen eines Startsignals
		case ERROR_START_CONDITION:
				LcdWriteString("StartRead",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler beim Erzeugen eines wiederholten Startsignals
		case ERROR_REPEATED_START_CONDITION:
				LcdWriteString("RepeatedStart",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler bei der Adressierung eines Slaves/ Anwahl eines Slaves; write
		case ERROR_SLAVE_ADDRESS_WRITE:
				LcdWriteString("AddressWrite",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler bei der Adressierung eines Slaves/ Anwahl eines Slaves; read
		case ERROR_SLAVE_ADDRESS_READ:
				LcdWriteString("AddressRead",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler bei der Nutzdatenbertragung/ Senden des Datenbytes; write
		case ERROR_DATA_WRITE:
				LcdWriteString("DataWrite",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Fehler bei der Nutzdatenbertragung/ Senden des Datenbytes; read
		case ERROR_DATA_READ:
				LcdWriteString("DataRead",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);
				break;
		// Unbekannter Fehler bzw. Programmfehler
		default:
				LcdWriteString("Unknown",
											 ERROR_OUTPUT_LINE,
											 ERROR_OUTPUT_COLUMN);			
	}
}